#!/usr/bin/env bash
# run as root
export euid=$(id -u)
if [ ${euid} -eq 0 ]; then
    echo "Running as root"
else
    echo "This script should be run as root"
    exit
fi

# check if password was set
if [[ -z "${BORG_PASSPHRASE}" ]]; then
    echo 'BORG_PASSPHRASE is not set, `export BORG_PASSPHRASE=PASS`'
    exit
fi
export REMOTE_USER=fileserver@homeserver.box
export REMOTE_PATH=/srv/data/backup/borg/theseus
export BORG_REPO=$REMOTE_USER:$REMOTE_PATH
export BORG_BIN="$(which borg)"

# some helpers and error handling:
info() { printf "\n%s %s\n\n" "$( date )" "$*" >&2; }
trap 'echo $( date ) Backup interrupted >&2; exit 2' INT TERM

echo "Starting backup"

# Backup the most important directories into an archive named after
# the machine this script is currently running on:

$BORG_BIN create                         \
    --verbose                       \
    --filter AME                    \
    --list                          \
    --stats                         \
    --show-rc                       \
    --compression zstd,11           \
    --exclude-caches                \
    --exclude 'home/*/.cache/*'     \
    --exclude 'home/*/Documents/code/compile/*'     \
    --exclude '*Cache*'     \
    --exclude 'home/*/Download/*'     \
    --exclude 'home/*/.local/*'     \
    --exclude '*JetBrains/Toolbox/apps*'     \
    --exclude '*/.Trash*'           \
    --exclude '*/Trash*'           \
    --exclude '*/build/*'           \
    --exclude '*Steam*'           \
                                    \
    ${BORG_REPO}::'{hostname}-{now}'            \
    /home                           \
    /root                           \
    /etc

backup_exit=$?

echo "Pruning repository"

# Use the `prune` subcommand to maintain 3 daily, 3 weekly and 6 monthly
# archives of THIS machine. The '{hostname}-*' matching is very important to
# limit prune's operation to this machine's archives and not apply to
# other machines' archives also:

$BORG_BIN prune                          \
    --list                          \
    --glob-archives '{hostname}-*'  \
    --show-rc                       \
    --keep-daily    3               \
    --keep-weekly   3               \
    --keep-monthly  6

prune_exit=$?

# actually free repo disk space by compacting segments

echo "Compacting repository"

$BORG_BIN compact

compact_exit=$?

# use highest exit code as global exit code
global_exit=$(( backup_exit > prune_exit ? backup_exit : prune_exit ))
global_exit=$(( compact_exit > global_exit ? compact_exit : global_exit ))

if [ ${global_exit} -eq 0 ]; then
    echo "Backup, Prune, and Compact finished successfully"
elif [ ${global_exit} -eq 1 ]; then
    echo "Backup, Prune, and/or Compact finished with warnings"
else
    echo "Backup, Prune, and/or Compact finished with errors"
fi

ssh $REMOTE_USER du -hs $REMOTE_PATH

exit ${global_exit}
